@@ -3,7 +3,7 @@
 #######################################################################
 #  Copyright (C) 2020 Vinh Tran
 #
-#  Calculate FAS cutoff for each core ortholog group of the core set
+#  Create phylogenetic profile outputs from the ortholog search result
 #
 #  This script is distributed in the hope that it will be useful,
 #  but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -27,38 +27,11 @@ import glob
 import multiprocessing as mp
 from tqdm import tqdm
 from greedyFAS.mainFAS import fasInput
-
-def checkFileExist(file, msg):
-    if not os.path.exists(os.path.abspath(file)):
-        sys.exit('%s not found! %s' % (file, msg))
-
-def roundTo4(number):
-    return("%.4f" % round(float(number), 4))
-
-def readFile(file):
-    if os.path.exists(file):
-        with open(file, 'r') as f:
-            lines = f.read().splitlines()
-            f.close()
-            return(lines)
-    else:
-        sys.exit('%s not found' % file)
-
-def removeDup(file):
-    if os.path.exists(file):
-        lines_seen = set() # holds lines already seen
-        outfile = open(file+'.temp', 'w')
-        for line in open(file, 'r'):
-            if line not in lines_seen: # not a duplicate
-                if not len(line.strip()) == 0: # not empty
-                    outfile.write(line)
-                    lines_seen.add(line)
-        outfile.close()
-        os.replace(file+'.temp', file)
+import fcat.functions as fcatFn
 
 def readRefspecFile(refspecFile):
     groupRefspec = {}
-    for line in readFile(refspecFile):
+    for line in fcatFn.readFile(refspecFile):
         groupRefspec[line.split('\t')[0]] = line.split('\t')[1]
     return(groupRefspec)
 
@@ -80,9 +53,14 @@ def createProfile1(coreDir, outDir, coreSet, queryID, force, groupRefspec):
         # output files
         finalFa = open('%s/%s.mod.fa' % (phyloprofileDir, queryID), 'w')
         finalPhyloprofile = open('%s/%s_mode1.phyloprofile' % (phyloprofileDir, queryID), 'w')
-        finalPhyloprofile.write('geneID\tncbiID\torthoID\tFAS\n')
+        lines_seen = set() # holds lines already seen
+        header = 'geneID\tncbiID\torthoID\tFAS\n'
+        finalPhyloprofile.write(header)
+        lines_seen.add(header)
         finalLen = open('%s/%s_length.phyloprofile' % (phyloprofileDir, queryID), 'w')
+        linesLen_seen = set() # holds lines already seen
         finalLen.write('geneID\tncbiID\torthoID\tLength\n')
+        linesLen_seen.add(header)
         # parse into phyloprofile files
         fdogOutDir = '%s/fcatOutput/%s/%s/fdogOutput' % (outDir, coreSet, queryID)
         mergedFa = '%s/%s_all.extended.fa' % (fdogOutDir, queryID)
@@ -90,7 +68,7 @@ def createProfile1(coreDir, outDir, coreSet, queryID, force, groupRefspec):
         groupScoreFwd = {} # all fwd fas scores of query ortholog vs core proteins
         groupScoreRev = {} # all rev fas scores of query ortholog vs core proteins
         groupOrtho = {} # query ortholog ID of each group, used for orthoID column of phyloprofile output
-        for line in readFile('%s/%s_all.phyloprofile' % (fdogOutDir, queryID)):
+        for line in fcatFn.readFile('%s/%s_all.phyloprofile' % (fdogOutDir, queryID)):
             if not line.split('\t')[0] == 'geneID':
                 groupID = line.split('\t')[0]
                 if not groupID in groupScoreFwd:
@@ -106,20 +84,26 @@ def createProfile1(coreDir, outDir, coreSet, queryID, force, groupRefspec):
         for groupID in groupOrtho:
             groupIDmod = '_'.join(groupID.split('_')[1:])
             groupOrthoMod = '_'.join(groupOrtho[groupID].split('_')[1:])
-            newline = '%s\t%s\t%s\t%s\n' % (groupIDmod, 'ncbi' + str(queryID.split('@')[1]), groupOrthoMod, roundTo4(statistics.mean((statistics.mean(groupScoreFwd[groupID]), statistics.mean(groupScoreRev[groupID])))))
-            finalPhyloprofile.write(newline)
+            newline = '%s\t%s\t%s\t%s\n' % (groupIDmod, 'ncbi' + str(queryID.split('@')[1]), groupOrthoMod, fcatFn.roundTo4(statistics.mean((statistics.mean(groupScoreFwd[groupID]), statistics.mean(groupScoreRev[groupID])))))
+            if newline not in lines_seen: # not a duplicate
+                finalPhyloprofile.write(newline)
+                lines_seen.add(newline)
             # append profile of core sequences for this group
             meanCoreFile = '%s/core_orthologs/%s/%s/fas_dir/cutoff_dir/2.cutoff' % (coreDir, coreSet, groupIDmod)
-            for tax in readFile(meanCoreFile):
+            for tax in fcatFn.readFile(meanCoreFile):
                 if not tax.split('\t')[0] == 'taxa':
                     # not include core taxon that have the same taxonomy ID as query
                     if queryID.split('@')[1] == groupRefspec[groupIDmod].split('@')[1]:
                         if not tax.split('\t')[0] == groupRefspec[groupIDmod]:
-                            ppCore = '%s\t%s\t%s|1\t%s\n' % (groupIDmod, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2].strip(), roundTo4(tax.split('\t')[1]))
-                            finalPhyloprofile.write(ppCore)
+                            ppCore = '%s\t%s\t%s|1\t%s\n' % (groupIDmod, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2].strip(), fcatFn.roundTo4(float(tax.split('\t')[1])))
+                            if ppCore not in lines_seen: # not a duplicate
+                                finalPhyloprofile.write(ppCore)
+                                lines_seen.add(ppCore)
                     else:
-                        ppCore = '%s\t%s\t%s|1\t%s\n' % (groupIDmod, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2].strip(), roundTo4(tax.split('\t')[1]))
-                        finalPhyloprofile.write(ppCore)
+                        ppCore = '%s\t%s\t%s|1\t%s\n' % (groupIDmod, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2].strip(), fcatFn.roundTo4(float(tax.split('\t')[1])))
+                        if ppCore not in lines_seen: # not a duplicate
+                            finalPhyloprofile.write(ppCore)
+                            lines_seen.add(ppCore)
         # length phyloprofile file and final fasta file
         checkFa = []
         for s in SeqIO.parse(mergedFa, 'fasta'):
@@ -130,47 +114,60 @@ def createProfile1(coreDir, outDir, coreSet, queryID, force, groupRefspec):
                         finalFa.write('>%s\n%s\n' % (idMod, s.seq))
                         checkFa.append(idMod)
                     ppLen = '%s\t%s\t%s\t%s\n' % (idMod.split('|')[0], 'ncbi' + str(idMod.split('|')[1].split('@')[1]), idMod, len(s.seq))
-                    finalLen.write(ppLen)
+                    if ppLen not in linesLen_seen: # not a duplicate
+                        finalLen.write(ppLen)
+                        linesLen_seen.add(ppLen)
             else:
                 if not idMod in checkFa:
                     finalFa.write('>%s\n%s\n' % (idMod, s.seq))
                     checkFa.append(idMod)
                 ppLen = '%s\t%s\t%s\t%s\n' % (idMod.split('|')[0], 'ncbi' + str(idMod.split('|')[1].split('@')[1]), idMod, len(s.seq))
-                finalLen.write(ppLen)
+                if ppLen not in linesLen_seen: # not a duplicate
+                    finalLen.write(ppLen)
+                    linesLen_seen.add(ppLen)
         # add missing groups
         if os.path.exists('%s/fcatOutput/%s/%s/missing.txt' % (outDir, coreSet, queryID)):
-            for missingGr in readFile('%s/fcatOutput/%s/%s/missing.txt' % (outDir, coreSet, queryID)):
+            for missingGr in fcatFn.readFile('%s/fcatOutput/%s/%s/missing.txt' % (outDir, coreSet, queryID)):
                 # add to mode2 profile
                 meanCoreFile = '%s/core_orthologs/%s/%s/fas_dir/cutoff_dir/2.cutoff' % (coreDir, coreSet, missingGr)
-                for tax in readFile(meanCoreFile):
+                for tax in fcatFn.readFile(meanCoreFile):
                     if not tax.split('\t')[0] == 'taxa':
                         # not include core taxon that have the same taxonomy ID as query
                         if queryID.split('@')[1] == groupRefspec[missingGr].split('@')[1]:
                             if not tax.split('\t')[0] == groupRefspec[missingGr]:
-                                ppCore = '%s\t%s\t%s|1\t%s\n' % (missingGr, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2], roundTo4(tax.split('\t')[1]))
-                                finalPhyloprofile.write(ppCore)
+                                ppCore = '%s\t%s\t%s|1\t%s\n' % (missingGr, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2], fcatFn.roundTo4(tax.split('\t')[1]))
+                                if ppCore not in lines_seen: # not a duplicate
+                                    finalPhyloprofile.write(ppCore)
+                                    lines_seen.add(ppCore)
                         else:
-                            ppCore = '%s\t%s\t%s|1\t%s\n' % (missingGr, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2], roundTo4(tax.split('\t')[1]))
-                            finalPhyloprofile.write(ppCore)
+                            ppCore = '%s\t%s\t%s|1\t%s\n' % (missingGr, 'ncbi' + str(tax.split('\t')[0].split('@')[1]), tax.split('\t')[2], fcatFn.roundTo4(tax.split('\t')[1]))
+                            if ppCore not in lines_seen: # not a duplicate
+                                finalPhyloprofile.write(ppCore)
+                                lines_seen.add(ppCore)
                 # add to length profile and fasta file
                 faCoreFile = '%s/core_orthologs/%s/%s/%s.fa' % (coreDir, coreSet, missingGr, missingGr)
                 for sm in SeqIO.parse(faCoreFile, 'fasta'):
                     finalFa.write('>%s\n%s\n' % (sm.id+'|1', sm.seq))
                     ppLen = '%s\t%s\t%s\t%s\n' % (sm.id.split('|')[0], 'ncbi' + str(sm.id.split('|')[1].split('@')[1]), sm.id+'|1', len(sm.seq))
-                    finalLen.write(ppLen)
+                    if ppLen not in linesLen_seen: # not a duplicate
+                        finalLen.write(ppLen)
+                        linesLen_seen.add(ppLen)
         finalPhyloprofile.close()
         finalFa.close()
         finalLen.close()
         # delete duplicate lines
-        removeDup('%s/%s_mode1.phyloprofile' % (phyloprofileDir, queryID))
-        removeDup('%s/%s_length.phyloprofile' % (phyloprofileDir, queryID))
+        # removeDup('%s/%s_mode1.phyloprofile' % (phyloprofileDir, queryID))
+        # removeDup('%s/%s_length.phyloprofile' % (phyloprofileDir, queryID))
 
 def createProfile23(coreDir, outDir, coreSet, queryID, force):
     # output files
     (mode, phyloprofileDir) = outputMode(outDir, coreSet, queryID, force, queryID+'_mode2.phyloprofile')
     if not mode == 0:
         finalPhyloprofile = open('%s/%s_mode2.phyloprofile' % (phyloprofileDir, queryID), 'w')
-        finalPhyloprofile.write('geneID\tncbiID\torthoID\tFAS\n')
+        lines_seen = set() # holds lines already seen
+        header = 'geneID\tncbiID\torthoID\tFAS\n'
+        finalPhyloprofile.write(header)
+        lines_seen.add(header)
         # parse into phyloprofile file
         fdogOutDir = '%s/fcatOutput/%s/%s/fdogOutput' % (outDir, coreSet, queryID)
         out = os.listdir(fdogOutDir)
@@ -180,42 +177,49 @@ def createProfile23(coreDir, outDir, coreSet, queryID, force):
                 groups = os.listdir(refDir)
                 # move to phyloprofile output dir
                 if os.path.exists('%s/%s.phyloprofile' % (refDir, refSpec)):
-                    for line in readFile('%s/%s.phyloprofile' % (refDir, refSpec)):
+                    for line in fcatFn.readFile('%s/%s.phyloprofile' % (refDir, refSpec)):
                         if queryID in line:
                             tmpQuery = line.split('\t')
                             # statistics.mean((float(line.split('\t')[3], float(line.split('\t')[4]))
-                            finalPhyloprofile.write('%s\t%s\t%s\t%s\n' % (tmpQuery[0], tmpQuery[1], tmpQuery[2], roundTo4(statistics.mean((float(line.split('\t')[3]), float(line.split('\t')[4]))))))
+                            line = '%s\t%s\t%s\t%s\n' % (tmpQuery[0], tmpQuery[1], tmpQuery[2], fcatFn.roundTo4(statistics.mean((float(line.split('\t')[3]), float(line.split('\t')[4])))))
+                            if line not in lines_seen: # not a duplicate
+                                finalPhyloprofile.write(line)
+                                lines_seen.add(line)
                 # append profile of core sequences
                 for groupID in groups:
                     coreFasDir = '%s/core_orthologs/%s/%s/fas_dir/fasscore_dir' % (coreDir, coreSet, groupID)
                     for fasFile in glob.glob('%s/*.tsv' % coreFasDir):
                         if queryID.split('@')[1] == refSpec.split('@')[1]:
                             if not refSpec in fasFile:
-                                for fLine in readFile(fasFile):
+                                for fLine in fcatFn.readFile(fasFile):
                                     if refSpec in fLine.split('\t')[0]:
                                         tmp = fLine.split('\t')
                                         revFAS = 0
                                         revFile = '%s/%s.tsv' % (coreFasDir, tmp[0].split('|')[1])
-                                        for revLine in readFile(revFile):
+                                        for revLine in fcatFn.readFile(revFile):
                                             if tmp[1] == revLine.split('\t')[0]:
                                                 revFAS = revLine.split('\t')[2].split('/')[0]
-                                        coreLine = '%s\t%s\t%s\t%s\n' % (groupID, 'ncbi' + str(tmp[1].split('|')[1].split('@')[1]), tmp[1]+'|1', roundTo4(statistics.mean((float(tmp[2].split('/')[0]), float(revFAS)))))
-                                        finalPhyloprofile.write(coreLine)
+                                        coreLine = '%s\t%s\t%s\t%s\n' % (groupID, 'ncbi' + str(tmp[1].split('|')[1].split('@')[1]), tmp[1]+'|1', fcatFn.roundTo4(statistics.mean((float(tmp[2].split('/')[0]), float(revFAS)))))
+                                        if coreLine not in lines_seen: # not a duplicate
+                                            finalPhyloprofile.write(coreLine)
+                                            lines_seen.add(coreLine)
                         else:
-                            for fLine in readFile(fasFile):
+                            for fLine in fcatFn.readFile(fasFile):
                                 if refSpec in fLine.split('\t')[0]:
                                     tmp = fLine.split('\t')
                                     revFAS = 0
                                     revFile = '%s/%s.tsv' % (coreFasDir, tmp[0].split('|')[1])
-                                    for revLine in readFile(revFile):
+                                    for revLine in fcatFn.readFile(revFile):
                                         if tmp[1] == revLine.split('\t')[0]:
                                             revFAS = revLine.split('\t')[2].split('/')[0]
-                                    coreLine = '%s\t%s\t%s\t%s\n' % (groupID, 'ncbi' + str(tmp[1].split('|')[1].split('@')[1]), tmp[1]+'|1', roundTo4(statistics.mean((float(tmp[2].split('/')[0]), float(revFAS)))))
-                                    finalPhyloprofile.write(coreLine)
+                                    coreLine = '%s\t%s\t%s\t%s\n' % (groupID, 'ncbi' + str(tmp[1].split('|')[1].split('@')[1]), tmp[1]+'|1', fcatFn.roundTo4(statistics.mean((float(tmp[2].split('/')[0]), float(revFAS)))))
+                                    if coreLine not in lines_seen: # not a duplicate
+                                        finalPhyloprofile.write(coreLine)
+                                        lines_seen.add(coreLine)
         # finalize
         finalPhyloprofile.close()
         # delete duplicate lines
-        removeDup('%s/%s_mode2.phyloprofile' % (phyloprofileDir, queryID))
+        # removeDup('%s/%s_mode2.phyloprofile' % (phyloprofileDir, queryID))
         shutil.copy('%s/%s_mode2.phyloprofile' % (phyloprofileDir, queryID), '%s/%s_mode3.phyloprofile' % (phyloprofileDir, queryID))
 
 def getDomain(args):
@@ -237,12 +241,11 @@ def createDomainFile(coreDir, outDir, coreSet, queryID, refspecFile, ppFile, cpu
     # output files
     (mode, phyloprofileDir) = outputMode(outDir, coreSet, queryID, force, queryID+'.domains')
     if not mode == 0:
-        finalDomain = open('%s/fcatOutput/%s/%s/phyloprofileOutput/%s.domains' % (outDir, coreSet, queryID, queryID), 'w')
         # get refspec for each group
         refTaxid = readRefspecFile(refspecFile)
         # get parsing job
         domainJobs = []
-        for line in readFile(ppFile):
+        for line in fcatFn.readFile(ppFile):
             if not "geneID" in line:
                 groupID = line.split('\t')[2].split('|')[0]
                 # get ortho ID
@@ -256,7 +259,7 @@ def createDomainFile(coreDir, outDir, coreSet, queryID, refspecFile, ppFile, cpu
                     domainJobs.append((jsonFile, groupID, line.split('\t')[2], '%s|%s|%s' % (groupID, taxID, orthoID), line.split('\t')[2]))
                 # get prot ID for refspec
                 cutoff2File = '%s/core_orthologs/%s/%s/fas_dir/cutoff_dir/2.cutoff' % (coreDir, coreSet, groupID)
-                for ll in readFile(cutoff2File):
+                for ll in fcatFn.readFile(cutoff2File):
                     if refTaxid[groupID] in ll:
                         groupJsonFile = '%s/core_orthologs/%s/%s/fas_dir/annotation_dir/%s.json' % (coreDir, coreSet, groupID, groupID)
                         domainJobs.append((groupJsonFile, groupID, line.split('\t')[2], ll.split('\t')[-1], ll.split('\t')[-1]+'|1'))
@@ -269,9 +272,10 @@ def createDomainFile(coreDir, outDir, coreSet, queryID, refspecFile, ppFile, cpu
         pool.close()
         pool.join()
         # write domain output
-        finalDomain.write('%s\n' % '\n'.join(domainOut))
-        finalDomain.close()
-        removeDup('%s/fcatOutput/%s/%s/phyloprofileOutput/%s.domains' % (outDir, coreSet, queryID, queryID))
+        with open('%s/fcatOutput/%s/%s/phyloprofileOutput/%s.domains' % (outDir, coreSet, queryID, queryID), "w") as finalDomain:
+            value = '\n'.join(set(list(filter(None, domainOut))))
+            finalDomain.write('%s\n' % value)
+        # removeDup('%s/fcatOutput/%s/%s/phyloprofileOutput/%s.domains' % (outDir, coreSet, queryID, queryID))
 
 def deleteFolder(folder):
     if os.path.exists(folder):
@@ -283,7 +287,7 @@ def deleteFolder(folder):
 def createPhyloProfile(args):
     coreDir = os.path.abspath(args.coreDir)
     coreSet = args.coreSet
-    checkFileExist(coreDir + '/core_orthologs/' + coreSet, '')
+    fcatFn.checkFileExist(coreDir + '/core_orthologs/' + coreSet, '')
     queryID = args.queryID
     outDir = args.outDir
     if outDir == '':
@@ -309,7 +313,7 @@ def createPhyloProfile(args):
             createProfile1(coreDir, outDir, coreSet, queryID, force, groupRefspec)
             createProfile23(coreDir, outDir, coreSet, queryID, force)
             if args.noDomain == False:
-                createDomainFile(coreDir, outDir, coreSet, queryID, '%s/last_refspec.txt' % fcatOut, '%s/phyloprofileOutput/%s_mode1.phyloprofile' % (fcatOut, queryID), args.cpus, args.force)
+                createDomainFile(coreDir, outDir, coreSet, queryID, '%s/last_refspec.txt' % fcatOut, '%s/phyloprofileOutput/%s_mode1.phyloprofile' % (fcatOut, queryID), args.cpus, force)
         else:
             sys.exit('No last_refspec.txt file found!')
 
@@ -319,7 +323,7 @@ def createPhyloProfile(args):
                 shutil.rmtree('%s/fdogOutput/' % (fcatOut))
 
 def main():
-    version = '0.0.23'
+    version = '0.0.26'
     parser = argparse.ArgumentParser(description='You are running fcat version ' + str(version) + '.')
     required = parser.add_argument_group('required arguments')
     optional = parser.add_argument_group('optional arguments')
@@ -327,14 +331,14 @@ def main():
     required.add_argument('-c', '--coreSet', help='Name of core set, which is subfolder within coreDir/core_orthologs/ directory', action='store', default='', required=True)
     required.add_argument('--queryID', help='ID of taxon of interest (e.g. HUMAN@9606@3)', action='store', default='', type=str)
     optional.add_argument('-o', '--outDir', help='Path to output directory', action='store', default='')
-    optional.add_argument('--force', help='Force overwrite existing data', action='store_true', default=False)
+    optional.add_argument('--forceProfile', help='Force overwrite existing phylogenetic profile data', action='store_true', default=False)
     optional.add_argument('--noDomain', help='Not output feature domain files', action='store_true', default=False)
     optional.add_argument('--keep', help='Keep temporary phyloprofile data', action='store_true', default=False)
     optional.add_argument('--cpus', help='Number of CPUs used for annotation. Default = 4', action='store', default=4, type=int)
     args = parser.parse_args()
 
     start = time.time()
-    searchOrtho(args)
+    createPhyloProfile(args)
     ende = time.time()
     print('Finished in ' + '{:5.3f}s'.format(ende-start))
 
